@@ -24,6 +24,10 @@
 #' dispatching on `to`: developers may implement their own S3 methods for
 #' custom vector types.
 #'
+#' Note that unregistered extension types will by default issue a warning.
+#' Use `options(nanoarrow.warn_unregistered_extension = FALSE)` to disable
+#' this behaviour.
+#'
 #' @param array A [nanoarrow_array][as_nanoarrow_array].
 #' @param to A target prototype object describing the type to which `array`
 #'   should be converted, or `NULL` to use the default conversion as
@@ -64,6 +68,8 @@
 #'   be converted to [blob::blob()].
 #' - [vctrs::list_of()]: List, large list, and fixed-size list types can be
 #'   converted to [vctrs::list_of()].
+#' - [matrix()]: Fixed-size list types can be converted to
+#'   `matrix(ptype, ncol = fixed_size)`.
 #' - [data.frame()]: Struct types can be converted to [data.frame()].
 #' - [vctrs::unspecified()]: Any type can be converted to [vctrs::unspecified()];
 #'   however, a warning will be raised if any non-null values are encountered.
@@ -140,23 +146,13 @@ convert_fallback_other <- function(array, offset, length, to) {
 }
 
 #' @export
-convert_array.double <- function(array, to, ...) {
-  # Handle conversion from decimal128 via arrow
-  schema <- infer_nanoarrow_schema(array)
-  parsed <- nanoarrow_schema_parse(schema)
-  if (parsed$type == "decimal128") {
-    assert_arrow_installed(
-      sprintf(
-        "convert %s array to object of type double",
-        nanoarrow_schema_formatted(schema)
-      )
-    )
-
-    arrow_array <- as_arrow_array.nanoarrow_array(array)
-    arrow_array$as_vector()
-  } else {
-    NextMethod()
+convert_array.nanoarrow_vctr <- function(array, to, ...) {
+  schema <- attr(to, "schema", exact = TRUE)
+  if (is.null(schema)) {
+    schema <- infer_nanoarrow_schema(array)
   }
+
+  new_nanoarrow_vctr(list(array), schema, class(to))
 }
 
 #' @export
