@@ -80,6 +80,8 @@ test_that("convert to vector works for data.frame", {
 })
 
 test_that("convert to vector works for partial_frame", {
+  skip_if_not_installed("vctrs")
+
   array <- as_nanoarrow_array(
     data.frame(a = 1L, b = "two", stringsAsFactors = FALSE)
   )
@@ -115,6 +117,8 @@ test_that("convert to vector works for dictionary<struct> -> data.frame()", {
 })
 
 test_that("convert to vector works for function()", {
+  skip_if_not_installed("tibble")
+
   tibble_or_bust <- function(array, ptype) {
     if (is.data.frame(ptype)) {
       ptype <- tibble::as_tibble(ptype)
@@ -135,6 +139,8 @@ test_that("convert to vector works for function()", {
 })
 
 test_that("convert to vector works for tibble", {
+  skip_if_not_installed("tibble")
+
   array <- as_nanoarrow_array(
     data.frame(a = 1L, b = "two", stringsAsFactors = FALSE)
   )
@@ -160,6 +166,121 @@ test_that("convert to vector works for tibble", {
   )
 })
 
+test_that("convert to vector works for nanoarrow_vctr()", {
+  array <- as_nanoarrow_array(c("one", "two", "three"))
+
+  # Check implicit/inferred nanoarrow_vctr() schema
+  vctr <- convert_array(array, nanoarrow_vctr())
+  expect_s3_class(vctr, "nanoarrow_vctr")
+  expect_length(vctr, 3)
+  schema <- infer_nanoarrow_schema(vctr)
+  expect_identical(schema$format, "u")
+
+  # Check with explicit schema of the correct type
+  vctr <- convert_array(array, nanoarrow_vctr(na_string()))
+  expect_s3_class(vctr, "nanoarrow_vctr")
+  expect_length(vctr, 3)
+  schema <- infer_nanoarrow_schema(vctr)
+  expect_identical(schema$format, "u")
+
+  # Check conversion of a struct array
+  df <- data.frame(x = c("one", "two", "three"))
+  array <- as_nanoarrow_array(df)
+
+  vctr <- convert_array(array, nanoarrow_vctr())
+  expect_s3_class(vctr, "nanoarrow_vctr")
+  expect_length(vctr, 3)
+  schema <- infer_nanoarrow_schema(vctr)
+  expect_identical(schema$format, "+s")
+
+  vctr <- convert_array(array, nanoarrow_vctr(na_struct(list(x = na_string()))))
+  expect_s3_class(vctr, "nanoarrow_vctr")
+  expect_length(vctr, 3)
+  schema <- infer_nanoarrow_schema(vctr)
+  expect_identical(schema$format, "+s")
+})
+
+test_that("batched convert to vector works for nanoarrow_vctr()", {
+  empty_stream <- basic_array_stream(list(), schema = na_string())
+  empty_vctr <- convert_array_stream(empty_stream, nanoarrow_vctr())
+  expect_length(empty_vctr, 0)
+  expect_identical(infer_nanoarrow_schema(empty_vctr)$format, "u")
+
+  stream1 <- basic_array_stream(list(c("one", "two", "three")))
+  vctr1 <- convert_array_stream(stream1, nanoarrow_vctr())
+  expect_length(vctr1, 3)
+
+  stream2 <- basic_array_stream(
+    list(c("one", "two", "three"), c("four", "five", "six", "seven"))
+  )
+  vctr2 <- convert_array_stream(stream2, nanoarrow_vctr())
+  expect_length(vctr2, 7)
+  expect_identical(
+    convert_array_stream(as_nanoarrow_array_stream(vctr2)),
+    c("one", "two", "three", "four", "five", "six", "seven")
+  )
+})
+
+test_that("convert to vector works for data.frame(nanoarrow_vctr())", {
+  array <- as_nanoarrow_array(data.frame(x = 1:5))
+  df_vctr <- convert_array(array, data.frame(x = nanoarrow_vctr()))
+  expect_s3_class(df_vctr$x, "nanoarrow_vctr")
+  expect_identical(
+    convert_array_stream(as_nanoarrow_array_stream(df_vctr$x)),
+    1:5
+  )
+})
+
+test_that("convert to vector works for list_of(nanoarrow_vctr())", {
+  skip_if_not_installed("arrow")
+  skip_if_not_installed("vctrs")
+
+  array <- as_nanoarrow_array(
+    list(1:5, 6:10, NULL, 11:13),
+    schema = na_list(na_int32())
+  )
+
+  list_vctr <- convert_array(array, vctrs::list_of(nanoarrow_vctr()))
+
+  # Each item in the list should be a vctr with one chunk that is a slice
+  # of the original array
+  expect_s3_class(list_vctr[[1]], "nanoarrow_vctr")
+  vctr_array <- attr(list_vctr[[1]], "chunks")[[1]]
+  expect_identical(vctr_array$offset, 0L)
+  expect_identical(vctr_array$length, 5L)
+  expect_identical(convert_buffer(vctr_array$buffers[[2]]), 1:5)
+
+  expect_s3_class(list_vctr[[2]], "nanoarrow_vctr")
+  vctr_array <- attr(list_vctr[[2]], "chunks")[[1]]
+  expect_identical(vctr_array$offset, 5L)
+  expect_identical(vctr_array$length, 5L)
+  expect_identical(convert_buffer(vctr_array$buffers[[2]]), 1:10)
+
+  expect_null(list_vctr[[3]])
+
+  expect_s3_class(list_vctr[[4]], "nanoarrow_vctr")
+  vctr_array <- attr(list_vctr[[4]], "chunks")[[1]]
+  expect_identical(vctr_array$offset, 10L)
+  expect_identical(vctr_array$length, 3L)
+  expect_identical(convert_buffer(vctr_array$buffers[[2]]), 1:13)
+})
+
+test_that("batched convert to vector works for nanoarrow_vctr() keeps subclass", {
+  vctr_ptype <- nanoarrow_vctr(subclass = "some_subclass")
+
+  empty_stream <- basic_array_stream(list(), schema = na_string())
+  empty_vctr <- convert_array_stream(empty_stream, vctr_ptype)
+  expect_s3_class(empty_vctr, "some_subclass")
+
+  stream1 <- basic_array_stream(list(c("")))
+  vctr1 <- convert_array_stream(stream1, vctr_ptype)
+  expect_s3_class(vctr1, "some_subclass")
+
+  stream2 <- basic_array_stream(list(c(""), c("")))
+  vctr2 <- convert_array_stream(stream2, vctr_ptype)
+  expect_s3_class(vctr2, "some_subclass")
+})
+
 test_that("convert to vector works for struct-style vectors", {
   array <- as_nanoarrow_array(as.POSIXlt("2021-01-01", tz = "America/Halifax"))
   expect_identical(
@@ -178,6 +299,8 @@ test_that("convert to vector works for struct-style vectors", {
 })
 
 test_that("convert to vector works for unspecified()", {
+  skip_if_not_installed("vctrs")
+
   array <- nanoarrow_array_init(na_na())
   array$length <- 10
   array$null_count <- 10
@@ -364,6 +487,31 @@ test_that("convert to vector works for valid integer()", {
   )
 })
 
+test_that("convert to works for integer() -> character()", {
+  skip_if_not_installed("arrow")
+
+  arrow_int_types <- list(
+    int8 = arrow::int8(),
+    uint8 = arrow::uint8(),
+    int16 = arrow::int16(),
+    uint16 = arrow::uint16(),
+    int32 = arrow::int32(),
+    uint32 = arrow::uint32(),
+    int64 = arrow::int64()
+  )
+
+  ints <- c(NA, 0:10)
+  for (nm in names(arrow_int_types)) {
+    expect_identical(
+      convert_array(
+        as_nanoarrow_array(ints, schema = arrow_int_types[[!!nm]]),
+        character()
+      ),
+      as.character(ints)
+    )
+  }
+})
+
 test_that("convert to vector works for null -> logical()", {
   array <- nanoarrow_array_init(na_na())
   array$length <- 10
@@ -420,6 +568,7 @@ test_that("convert to vector works for valid double()", {
     uint32 = arrow::uint32(),
     int64 = arrow::int64(),
     uint64 = arrow::uint64(),
+    float16 = arrow::float16(),
     float32 = arrow::float32(),
     float64 = arrow::float64()
   )
@@ -464,22 +613,31 @@ test_that("convert to vector works for valid double()", {
   )
 })
 
-test_that("convert to vector works for decimal128 -> double()", {
+test_that("convert to vector works for decimal -> double()", {
+  constructors <- list(na_decimal32, na_decimal64, na_decimal256, na_decimal128)
+  for (constructor in constructors) {
+    numbers <- round(c(pi, -pi, 123.4567, NA, -123.4567, 0, 123), 4)
+
+    # Check scale of 4 (min required for lossless roundtrip)
+    array4 <- as_nanoarrow_array(numbers, schema = constructor(9, 4))
+    expect_identical(convert_array(array4), numbers)
+
+    # Check scale of 5 (requires adding some zeroes in C)
+    array5 <- as_nanoarrow_array(numbers, schema = constructor(9, 5))
+    expect_identical(convert_array(array5), numbers)
+
+    # Check negative scale (also requires adding some zeroes in C)
+    numbers_neg_scale <- c(12300, -12300, 0, NA, 100)
+    array_neg2 <- as_nanoarrow_array(numbers_neg_scale, schema = constructor(9, -2))
+    expect_identical(convert_array(array_neg2), numbers_neg_scale)
+  }
+
+  # Make sure we agree with arrow
   skip_if_not_installed("arrow")
 
-  array <- as_nanoarrow_array(arrow::Array$create(1:10)$cast(arrow::decimal128(20, 10)))
-
-  # Check via S3 dispatch
-  expect_equal(
-    convert_array(array, double()),
-    as.double(1:10)
-  )
-
-  # ...and via C -> S3 dispatch
-  expect_equal(
-    convert_array.default(array, double()),
-    as.double(1:10)
-  )
+  expect_identical(as.vector(arrow::as_arrow_array(array4)), numbers)
+  expect_identical(as.vector(arrow::as_arrow_array(array5)), numbers)
+  expect_identical(as.vector(arrow::as_arrow_array(array_neg2)), numbers_neg_scale)
 })
 
 test_that("convert to vector works for null -> double()", {
@@ -636,8 +794,22 @@ test_that("convert to vector works for character()", {
 
   # check an array that we can't convert
   expect_error(
-    convert_array(as_nanoarrow_array(1:5), character()),
-    "Can't convert array <int32> to R vector of type character"
+    convert_array(as_nanoarrow_array(1:5), list()),
+    "Can't convert array <int32> to R vector of type list"
+  )
+})
+
+test_that("convert to vector works for string_view -> character()", {
+  array <- as_nanoarrow_array(letters, schema = na_string_view())
+  expect_identical(
+    convert_array(array, character()),
+    letters
+  )
+
+  array_with_nulls <- as_nanoarrow_array(c(letters, NA), schema = na_string_view())
+  expect_identical(
+    convert_array(array_with_nulls, character()),
+    c(letters, NA)
   )
 })
 
@@ -709,6 +881,37 @@ test_that("convert to vector works for dictionary<string> -> factor()", {
   )
 })
 
+test_that("convert to vector works for decimal -> character()", {
+  constructors <- list(na_decimal32, na_decimal64, na_decimal256, na_decimal128)
+  for (constructor in constructors) {
+    numbers <- round(c(pi, -pi, 123.4567, NA, -123.4567, 0, 123), 4)
+
+    # Check scale of 4 (min required for lossless roundtrip)
+    array4 <- as_nanoarrow_array(numbers, schema = constructor(9, 4))
+    expect_identical(
+      convert_array(array4, character()),
+      c("3.1416", "-3.1416", "123.4567", NA_character_,
+        "-123.4567", "0.0000", "123.0000")
+    )
+
+    # Check scale of 5 (requires adding some zeroes in C)
+    array5 <- as_nanoarrow_array(numbers, schema = constructor(9, 5))
+    expect_identical(
+      convert_array(array5, character()),
+      c("3.14160", "-3.14160", "123.45670", NA_character_,
+        "-123.45670", "0.00000", "123.00000")
+    )
+
+    # Check negative scale (also requires adding some zeroes in C)
+    numbers_neg_scale <- c(12300, -12300, 0, NA, 100)
+    array_neg2 <- as_nanoarrow_array(numbers_neg_scale, schema = constructor(9, -2))
+    expect_identical(
+      convert_array(array_neg2, character()),
+      c("12300", "-12300", "000", NA_character_, "100")
+    )
+  }
+})
+
 test_that("batched convert to vector works for dictionary<string> -> factor()", {
   # A slightly different path: convert_array.factor() called from C multiple
   # times with different dictionaries each time.
@@ -753,7 +956,25 @@ test_that("convert to vector works for blob::blob()", {
   )
 })
 
+test_that("convert to vector works for binary_view -> blob::blob()", {
+  skip_if_not_installed("blob")
+
+  array <- as_nanoarrow_array(letters, schema = na_binary_view())
+  expect_identical(
+    convert_array(array, blob::blob()),
+    blob::as_blob(lapply(letters, charToRaw))
+  )
+
+  array_with_nulls <- as_nanoarrow_array(c(letters, NA), schema = na_binary_view())
+  expect_identical(
+    convert_array(array_with_nulls, blob::blob()),
+    blob::as_blob(c(lapply(letters, charToRaw), list(NULL)))
+  )
+})
+
 test_that("convert to vector works for null -> blob::blob()", {
+  skip_if_not_installed("blob")
+
   array <- nanoarrow_array_init(na_na())
   array$length <- 10
   array$null_count <- 10
@@ -766,6 +987,7 @@ test_that("convert to vector works for null -> blob::blob()", {
 
 test_that("convert to vector works for list -> vctrs::list_of", {
   skip_if_not_installed("arrow")
+  skip_if_not_installed("vctrs")
 
   array_list <- as_nanoarrow_array(
     arrow::Array$create(
@@ -788,7 +1010,7 @@ test_that("convert to vector works for list -> vctrs::list_of", {
 
   # With bad ptype
   expect_error(
-    convert_array(array_list, vctrs::list_of(.ptype = character())),
+    convert_array(array_list, vctrs::list_of(.ptype = list())),
     "Can't convert `item`"
   )
 
@@ -803,6 +1025,7 @@ test_that("convert to vector works for list -> vctrs::list_of", {
 
 test_that("convert to vector works for large_list -> vctrs::list_of", {
   skip_if_not_installed("arrow")
+  skip_if_not_installed("vctrs")
 
   array_list <- as_nanoarrow_array(
     arrow::Array$create(
@@ -825,13 +1048,14 @@ test_that("convert to vector works for large_list -> vctrs::list_of", {
 
   # With bad ptype
   expect_error(
-    convert_array(array_list, vctrs::list_of(.ptype = character())),
+    convert_array(array_list, vctrs::list_of(.ptype = list())),
     "Can't convert `item`"
   )
 })
 
 test_that("convert to vector works for fixed_size_list -> vctrs::list_of", {
   skip_if_not_installed("arrow")
+  skip_if_not_installed("vctrs")
 
   array_list <- as_nanoarrow_array(
     arrow::Array$create(
@@ -854,12 +1078,14 @@ test_that("convert to vector works for fixed_size_list -> vctrs::list_of", {
 
   # With bad ptype
   expect_error(
-    convert_array(array_list, vctrs::list_of(.ptype = character())),
+    convert_array(array_list, vctrs::list_of(.ptype = list())),
     "Can't convert `item`"
   )
 })
 
 test_that("convert to vector works for null -> vctrs::list_of()", {
+  skip_if_not_installed("vctrs")
+
   array <- nanoarrow_array_init(na_na())
   array$length <- 10
   array$null_count <- 10
@@ -870,6 +1096,31 @@ test_that("convert to vector works for null -> vctrs::list_of()", {
   )
 })
 
+test_that("convert to vector works for fixed_size_list_of() -> matrix()", {
+  mat <- matrix(1:6, ncol = 2, byrow = TRUE)
+  array <- as_nanoarrow_array(mat)
+
+  expect_identical(
+    convert_array(array, matrix(double(), ncol = 2)),
+    matrix(as.double(1:6), ncol = 2, byrow = TRUE)
+  )
+})
+
+test_that("convert to vector errors for invalid matrix()", {
+  expect_error(
+    convert_array(as_nanoarrow_array(1:6), matrix()),
+    "Can't convert array <int32> to R vector of type matrix"
+  )
+
+  mat <- matrix(1:6, ncol = 2, byrow = TRUE)
+  array <- as_nanoarrow_array(mat)
+  expect_error(
+    convert_array(array, matrix(integer(), ncol = 3)),
+    "Can't convert fixed_size_list(list_size=2) to matrix with 3 cols",
+    fixed = TRUE
+  )
+})
+
 test_that("convert to vector works for Date", {
   array_date <- as_nanoarrow_array(as.Date(c(NA, "2000-01-01")))
   expect_identical(
@@ -899,6 +1150,8 @@ test_that("convert to vector works for null -> Date", {
 })
 
 test_that("convert to vector works for hms", {
+  skip_if_not_installed("hms")
+
   array_time <- as_nanoarrow_array(hms::parse_hm("12:34"))
   expect_identical(
     convert_array(array_time),
@@ -907,6 +1160,8 @@ test_that("convert to vector works for hms", {
 })
 
 test_that("convert to vector works for null -> hms", {
+  skip_if_not_installed("hms")
+
   array <- nanoarrow_array_init(na_na())
   array$length <- 10
   array$null_count <- 10
@@ -1021,6 +1276,7 @@ test_that("convert to vector works for null -> difftime", {
 
 test_that("convert to vector works for data frames nested inside lists", {
   skip_if_not_installed("arrow")
+  skip_if_not_installed("vctrs")
 
   df_in_list <- vctrs::list_of(
     data.frame(x = 1:5),
@@ -1037,6 +1293,7 @@ test_that("convert to vector works for data frames nested inside lists", {
 
 test_that("convert to vector works for lists nested in data frames", {
   skip_if_not_installed("arrow")
+  skip_if_not_installed("vctrs")
 
   df_in_list_in_df <- data.frame(
     x = vctrs::list_of(
